@@ -71,6 +71,9 @@ void ObjP_raisePyExceptionInCocoa(PyGILState_STATE gilState)
 
 NSString* ObjP_str_p2o(PyObject *pStr)
 {
+    if (pStr == Py_None) {
+        return nil;
+    }
     OBJP_LOCKGIL;
     PyObject *pBytes = PyUnicode_AsUTF8String(pStr);
     OBJP_ERRCHECK(pBytes);
@@ -107,6 +110,23 @@ PyObject* ObjP_int_o2p(NSInteger i)
     return pResult;
 }
 
+CGFloat ObjP_float_p2o(PyObject *pFloat)
+{
+    OBJP_LOCKGIL;
+    CGFloat result = PyFloat_AsDouble(pFloat);
+    OBJP_UNLOCKGIL;
+    return result;
+}
+
+PyObject* ObjP_float_o2p(CGFloat f)
+{
+    OBJP_LOCKGIL;
+    PyObject *pResult = PyFloat_FromDouble(f);
+    OBJP_ERRCHECK(pResult);
+    OBJP_UNLOCKGIL;
+    return pResult;
+}
+
 BOOL ObjP_bool_p2o(PyObject *pBool)
 {
     OBJP_LOCKGIL;
@@ -138,6 +158,9 @@ NSObject* ObjP_obj_p2o(PyObject *pObj)
     else if (PyLong_Check(pObj)) {
         result = [NSNumber numberWithInt:ObjP_int_p2o(pObj)];
     }
+    else if (PyFloat_Check(pObj)) {
+        result = [NSNumber numberWithDouble:ObjP_float_p2o(pObj)];
+    }
     else if (PyBool_Check(pObj)) {
         result = [NSNumber numberWithBool:ObjP_bool_p2o(pObj)];
     }
@@ -164,7 +187,16 @@ PyObject* ObjP_obj_o2p(NSObject *obj)
         return ObjP_str_o2p((NSString *)obj);
     }
     else if ([obj isKindOfClass:[NSNumber class]]) {
-        return ObjP_int_o2p([(NSNumber *)obj intValue]);
+        const char *objc_type = [(NSNumber *)obj objCType];
+        if (objc_type[0] == 'c') {
+            return ObjP_bool_o2p([(NSNumber *)obj boolValue]);
+        }
+        else if ((objc_type[0] == 'f') || (objc_type[0] == 'd')) {
+            return ObjP_float_o2p([(NSNumber *)obj doubleValue]);
+        }
+        else {
+            return ObjP_int_o2p([(NSNumber *)obj intValue]);
+        }
     }
     else if ([obj isKindOfClass:[NSArray class]]) {
         return ObjP_list_o2p((NSArray *)obj);
@@ -179,6 +211,9 @@ PyObject* ObjP_obj_o2p(NSObject *obj)
 
 NSArray* ObjP_list_p2o(PyObject *pList)
 {
+    if (pList == Py_None) {
+        return nil;
+    }
     OBJP_LOCKGIL;
     PyObject *iterator = PyObject_GetIter(pList);
     OBJP_ERRCHECK(iterator);
@@ -186,7 +221,11 @@ NSArray* ObjP_list_p2o(PyObject *pList)
     NSMutableArray *result = [NSMutableArray array];
     while ( (item = PyIter_Next(iterator)) ) {
         OBJP_ERRCHECK(item);
-        [result addObject:ObjP_obj_p2o(item)];
+        NSObject *value = ObjP_obj_p2o(item);
+        if (value == nil) {
+            value = [NSNull null];
+        }
+        [result addObject:value];
         Py_DECREF(item);
     }
     Py_DECREF(iterator);
@@ -211,6 +250,9 @@ PyObject* ObjP_list_o2p(NSArray *list)
 
 NSDictionary* ObjP_dict_p2o(PyObject *pDict)
 {
+    if (pDict == Py_None) {
+        return nil;
+    }
     PyObject *pKey, *pValue;
     Py_ssize_t pos = 0;
     OBJP_LOCKGIL;
@@ -220,6 +262,9 @@ NSDictionary* ObjP_dict_p2o(PyObject *pDict)
         OBJP_ERRCHECK(pValue);
         NSString *key = ObjP_str_p2o(pKey);
         NSObject *value = ObjP_obj_p2o(pValue);
+        if (value == nil) {
+            value = [NSNull null];
+        }
         [result setObject:value forKey:key];
     }
     OBJP_UNLOCKGIL;
